
proc getFragments {} { ; #create a list of the fragments that were created by confirming matches
    global Clust NvCBCA Frags
    if {[info exists Frags]} {unset Frags}
    set Frags(nFrags) 0
    for {set i 0} {$i<=$Clust(nClust)} {incr i 1} {
	set Clust($i,visited) 0
    }
    for {set i 0} {$i<$Clust(nClust)} {incr i 1} {
	getFrags $i "regMode"
    }
}

proc getFrags {iClust mode} {
    global Clust NvCBCA Frags
    if {$mode != "forward"} {
	if { $Clust($iClust,visited) } { ; #don't investigate further if its already in a Fragment
	    return
	}
	set preClust [getClust $iClust pre] ; # set preClust to the cluster confirmed to be preceeding the current one
	if {$preClust > -1} {
	    getFrags $preClust "recur" ; # recursively get the preceeding clusters
	} else {
	    incr Frags(nFrags) 1
	    set Frags(current) $Frags(nFrags)
	}
	lappend Frags($Frags(current)) $iClust
	if { $mode == "recur" } {
	    set Clust($iClust,visited) 1
	    return
	}
    }
    set sucClust [getClust $iClust suc] ; # set sucClust to the cluster confirmed to be suceeding iClust
    set Clust($iClust,visited) 1 
    if {$sucClust > -1} {
	lappend Frags($Frags(current)) $sucClust
	getFrags $sucClust "forward"
    }
}

proc getClust {iClust dir} {
    global Clust Frags
    if {[info exists Clust($iClust,$dir)]} {
	foreach clust $Clust($iClust,$dir) {
	    set conf [lindex $clust 2]
	    if {$conf > 0} {
		return [lindex $clust 0]
	    }
	}
    }
} 

proc AssignFrag {filename } {
    global Nv_FragDisp ClustUsed
    set fileid [open $filename w]
    global Nv_Nresidues Fragment Seq Clust int_flag NvCBCA
    #    ZeroPeakLab $NvCBCA(0,intraResidue,peaks)
    #    ZeroPeakLab $NvCBCA(0,interResidue,peaks)
    #    ZeroPeakLab $NvCBCA(1,intraResidue,peaks)
    #    ZeroPeakLab $NvCBCA(1,interResidue,peaks)
    catch "unset ClustUsed"
    for {set i 0} {$i<$Clust(nClust)} {incr i} {
        set Clust($i,Seq) ""
    }
    if {[info exists Seq]} {unset Seq}
    set nFrag $Fragment(nFrag)
    set minRatio 0.095
    set iPass 1
    set ok 1
    SetupSequence
    set SortFrags ""
    for {set iFrag 0} {$iFrag<$nFrag} {incr iFrag} {
        set Fragment($iFrag,A) 0
	set Fragment($iFrag,seqPos) "none"
        set fragLength  [llength $Fragment($iFrag,clst)]
        if {$fragLength > 1} {
            lappend SortFrags "$iFrag $fragLength"
        }
    }
    set SortFrags [lsort -index 1 -integer -decreasing $SortFrags]
    while {$ok} {
        set ok 0
        for {set iSFrag 0} {$iSFrag<[llength $SortFrags]} {incr iSFrag} {
            if {$int_flag} {
                set int_flag 0
                close $fileid
                return
            }
            set iFrag [lindex [lindex $SortFrags $iSFrag] 0]
            if {[info exists Fragment($iFrag,use)] && !$Fragment($iFrag,use)} {continue}
            if {$Fragment($iFrag,A)} {continue}
            set used 0
            foreach clst $Fragment($iFrag,clst) link $Fragment($iFrag,link) {
                if {[info exists ClustUsed(${clst}_${link})] && $ClustUsed(${clst}_${link})} { 
                    set used 1 ; #if any of the clusters are already sequenced, skip this frag
                    break
                }
            }
            # puts "$iFrag $used"
            if {$used} {continue} ; #skip if already used
            set FragSize [llength $Fragment($iFrag,clst)]
            
            set nScores [llength $Fragment($iFrag,seqScores)]
            if {$nScores==0} {continue}
            set seqScore0 [lindex $Fragment($iFrag,seqScores) 0]
            set firstRes0 [lindex $seqScore0 0]
            set minScore0 [lindex $seqScore0 1]
            if {$nScores == 1} {
                if {[isAssigned $firstRes0 $FragSize]} {
                    continue
                }
                
                set totalScore $minScore0
                set minScore1 10000.0
                set firstRes1 0
                set ok 1
                set Ratio 0.0
            } else {
                set totalScore 0.0
                set firstRes0 0
                set firstRes1 0
                set minScore1 10000.0
                foreach seqScore $Fragment($iFrag,seqScores) {
                    set firstRes [lindex $seqScore 0]
                    if {[isAssigned $firstRes $FragSize]} {continue}
                    if {$firstRes0 == 0} {
                        set firstRes0 [lindex $seqScore 0]
                        set minScore0 [lindex $seqScore 1]
                    } else {
                        if {$firstRes1 == 0} {
                            set firstRes1 [lindex $seqScore 0]
                            set minScore1 [lindex $seqScore 1]
                        }
                    }
                    set totalScore [expr {$totalScore+[lindex $seqScore 1]}]
                }
                if {$firstRes0 == 0} {continue}
                set meanScore [expr {$totalScore/$nScores}]
                if {$meanScore == 0.0} {
                    set meanScore 1.0
                }
                
                if {$firstRes1 == 0} {
                    set Ratio 10.0
                } else {
                    set Ratio [expr {($minScore1-$minScore0)/$meanScore}]
                    
                }
                
            }
            set gWorst ""
            set minScore0 [expr {$minScore0/$FragSize}]
            set minScore1 [expr $minScore1/$FragSize]
            if {($totalScore < 100.0) && (($Ratio>$minRatio) || ($firstRes1==0))} {
                set Assigned 1
                set ok 1
                set Fragment($iFrag,A) 1
                if {[info exists Nv_FragDisp(canvas)] && [winfo exists $Nv_FragDisp(canvas)]} {
                    $Nv_FragDisp(canvas) itemconfigure Frag$iFrag -fill red
                }
                GetShifts $iFrag
                for {set iShift 0} {$iShift<$FragSize} {incr iShift} {
                    set iRes [expr {$iShift+$firstRes0}]
                    set Seq($iRes) 2
                    set bclst  [lindex $Fragment($iFrag,clst) $iShift]
                    set blink  [lindex $Fragment($iFrag,link) $iShift]
                    set clst  [lindex $bclst 0]
                    if {$clst != ""} {
                        set Seq($iRes,HN) $Clust($clst,HN)
                        set Seq($iRes,N) $Clust($clst,N)
                        lappend Clust($clst,Seq) $iRes
                    }
                    foreach aname $Fragment(AtmTypes) {
                        set ppm  [lindex $Fragment($iFrag,$aname) $iShift]
                        set Seq($iRes,$aname) $ppm
                    }
                    
                    set Seq($iRes,clst) $bclst
                    set Seq($iRes,link) $blink
                    set Seq($iRes,Frag) $iFrag
                    set Seq($iRes,FragElem) $iShift
                    #		    puts "$iFrag $iShift $iRes $Seq($iRes,HN) $Seq($iRes,N) $clst"
                }
                set iRes [expr {$FragSize+$firstRes0-1}]
                set Seq($iRes) 3
                set Seq($firstRes0) 1
            } else {
                set Assigned 0
            }
            set outs [format "%d %3d %3d %3d %3d %7.2f %3d %7.2f %5.3f   %s" $Assigned $iPass $iFrag $FragSize $firstRes0 $minScore0 $firstRes1 $minScore1 $Ratio $gWorst]
           # puts stdout $outs
            puts $fileid $outs
            if {$Assigned} {
                foreach clst $Fragment($iFrag,clst) link $Fragment($iFrag,link) {
                    set ClustUsed(${clst}_${link}) $iFrag
                }
		set Fragment($iFrag,seqPos) $firstRes0 ; #the position of this fragment in the seq 
            }
            flush $fileid
        }
        incr iPass
    }
    for {set iRes 1} {$iRes<=$Nv_Nresidues} {incr iRes} {
        if {$Seq($iRes,rname)=="PRO"} {
            puts -nonewline P
        } else {
            puts -nonewline stdout $Seq($iRes)
            puts -nonewline $fileid $Seq($iRes)
        }
        
        if { [expr {($iRes % 20)==0}]} {puts stdout ""}
    }
    puts stdout ""
    puts $fileid ""
    close $fileid
}

proc confirmAll {} {
    global Clust
    for {set i 0} {$i < $Clust(nClust)} {incr i 1} {
	foreach dir "suc pre" {
	    for {set j 0} {$j < [llength $Clust($i,$dir)]} {incr j} {
		set clust [lindex $Clust($i,$dir) $j]
		set clust [lreplace $clust 2 2 [expr 1 + $j]]
		set Clust($i,$dir) [lreplace $Clust($i,$dir) $j $j $clust]
		if {$dir == "suc"} {
		    lappend Clust($i,used) [lindex [lindex $Clust($i,suc) $j] 0]
		} elseif {[info exists Clust($i,used)]} {
		    set clustToFind [lindex [lindex $Clust($i,pre) $j] 0]
		    set found [lsearch $Clust($i,used) $clustToFind]
		    if {$found >= 0} {
			set score1 [lindex [lindex $Clust($i,suc) $found] 1] 
			set score2 [lindex [lindex $Clust($i,pre) $j] 1]
			if {$score1 > $score2} {
			    set clust [lindex $Clust($i,pre) $j]
			    set clust [lreplace $clust 2 2 0] 
			    set Clust($i,pre) [lreplace $Clust($i,pre) $j $j $clust]
			} else {
			    set clust [lindex $Clust($i,suc) $found]
			    set clust [lreplace $clust 2 2 0] 
			    set Clust($i,suc) [lreplace $Clust($i,suc) $found $found $clust]
			}
		    }
		}
	    }
	}
    }
}
